//*******************************************************
// 人間対戦チェスプログラム
// Copyright (C) 1993,1997,2013,2015 AIG-Soft
// V3.20以降専用
//*******************************************************
// オリジナルはX68のC言語版
// 対応：iPad 縦横、日英

// iPadのみ
if not checkCompatible(COMPATIBLE_IPAD) then stop

print "BASIC ver.";basicVersion$
print "BASIC Core ver.";basicCoreVersion$
print iOSInterfaceVersion$
if val(basicVersion$)<2.3 then {
	print "このバージョンでは動きません:ver.";basicVersion$
	// 配列周りのバグに引っかかるから
	end
}

// 定数
enum
// フィールドホーム座標
	SX	=90+30
	SY	=55

// 1ます表示サイズ
	MX	=48	://40
	MY	=48

// ます縦横個数（縦＝横）
	WW	=8

// 一応のフィールドエンド座標
	EX	=(SX+WW*MX)
	EY	=(SY+WW*MY)

// その枠座標
	FSX	=(SX-12)
	FSY	=(SY-13)
	FEX	=(EX+11)
	FEY	=(EY+12)

// Drawます座標
	ESX	=(EX+22)
	ESY	=(SY+2)
	EWX	=96
	EWY	=32
	EEX	=(ESX+EWX)
	EEY	=(ESY+EWY)

// チェックメイトます座標
	CHWY	=32
	CHWX	=96
	CHSX	=ESX
	CHSY	=(EEY+12)
	CHEX	=(CHSX+CHWX)
	CHEY	=(CHSY+CHWY)

// １手戻します座標
	TWX	=70
	TWY	=24
	TEY	=(FEY-12)
	TSX	=ESX
	TSY	=(TEY-TWY)
	TEX	=(TSX+TWX)

// ミニ表示１ます大きさ
	MMX	=6
	MMY	=6
// ミニ表示座標
	MSX	=(TSX+12)
	MSY	=(TSY-24-WW*MMY)

// 先手表示ます
	SGWX	=90
	STSX	=8
	STSY	=(EY-64)
	STEX	=(STSX+SGWX)
	STEY	=(STSY+44)

// 後手表示ます
	GTSX	=STSX
	GTSY	=(SY+16)
	GTEX	=(GTSX+SGWX)
	GTEY	=(GTSY+44)

// 先手チェック表示ます
	SCSX	=STSX
	SCSY	=(STEY+4)
	SCEX	=(SCSX+60)
	SCEY	=(SCSY+22)

// 後手チェック表示ます
	GCEY	=(GTSY-4)
	GCSX	=SCSX
	GCSY	=(GCEY-22)
	GCEX	=SCEX

// 先手勝利表示ます
	SWSX	=STSX
	SWSY	=(EY-72)
	SWEX	=(SWSX+60)
	SWEY	=(SWSY+76)

// 後手勝利表示ます
	GWSX	=SWSX
	GWSY	=GCSY
	GWEX	=(GWSX+60)
	GWEY	=(GWSY+76)

// 引き分け表示ます
	DSX	=STSX
	DSY	=(512/2-1-16-6)
	DEX	=(DSX+SGWX)
	DEY	=(DSY+44)

// 移動し直しメッセージ
	CSY1	=(FEY+4)	:// 先手
	CSY2	=(FSY-4-24)	:// 後手

// 手表示枠
	TEWX	=10		:// 手表示枠のｘ幅
	TEWY	=10		:// 手表示枠のｙ幅
	TESX	=ESX
	TESY	=(CHEY+16)
	TEEX	=(TESX+(TEWX+1)*6)
	TEEY	=(TESY+TEWY*12+4)

	SCRSX	=TESX+2
	SCRSY	=TESY+2
	SCRWX	=((TEEX-2)-SCRSX+1)
	SCRWY	=((TEEY-2)-(SCRSY+12)+1)

// メッセージ行
	MES_Y	=22

// キー
	KEY_NONE	=0
	SPACE		=' ':// hit any key
	KEY_DRAW	='D'
	KEY_MODOSHI	='T'
	KEY_CHECK	='C'
endenum

//*******************************************************
// masu(y,x)
//	bit
//	15	  : 移動可能ます : MV
//	14	  : Kingキャスリング移動可能ます : CSMV
//	13	  : とれる駒のあるます : GTMV
//	12	  : アンパッサンがあるので移動可能ます : APMV
//	11	  : アンパッサン可能Pawn（取られる方） : AP
//	10-8  : 予約
//	7	  : Pawn/King/Rook２手目以降 : SND
//	4,5,6 : Pawnの変化 : PCHG,bit=BPCHG
//			000=普通		, 001=----- , 010=Rook , 011=Bishop
//			100=Knight	, 101=Queen , 110=---- , 111=------
//	3	  : 先手=下(0),後手=上(1) : FSEN
//	2,1,0 : キャラクター : CHR,CHR or FSEN=ALCHR
//			000=なし		, 001=Pawn  , 010=Rook , 011=Bishop
//			100=Knight	, 101=Queen , 110=King , 111=------
//*******************************************************
// 駒番号

enum
	NONE	=0	:// 駒がない
	PAWN	=1
	ROOK	=2
	BISHOP	=3
	KNIGHT	=4
	QUEEN	=5
	KING	=6
	RESV	=7	:// 予約
endenum

//*******************************************************
// 駒移動・修飾情報
// 動ける位置

enum
	MV	=BIT15

// キャスリングで動ける位置
	CSMV	=BIT14

// 駒を取りに動ける位置
	GTMV	=BIT13

// アンパッサンされたＰａｗｎがあって動ける位置
	APMV	=BIT12

// 移動可能ます判定ビット
	MOVER	=(MV+CSMV+GTMV+APMV)
	notMOVER=(MOVER xor &hffff)	:// ビットクリア時の反転マスク

// アンパッサンされるＰａｗｎフラグ
	AP	=BIT11

// ２手目以降フラグ
	SND	=BIT7

// 先手／後手を入れ換えるビット
	FSEN	=BIT3

// masu(,)内からキャラクターを抜き出すandパターン
	CHR	=(BIT0+BIT1+BIT2)

// 相手を含むキャラクター
	ALCHR	=(CHR+FSEN)

// Pawnの変化ビットのある位置(bit番号)
	BPCHG	=4

// Pawnの変化ビット
	PCHG	=(CHR shl BPCHG)
endenum

//-------------------------------------------------------
// 表示キャラクター
dim str	 koma(7)={"","P","R","B","N","Q","K","?" }		:// 半角文字

// bitmapGet/Put用ワーク（プログラム全体で共通）
dim int buf(SCRWX*SCRWY)

// ますワークエリア
dim int	masu(WW,WW)	:// ます状態
dim int	kx(2),ky(2)	:// Ｋｉｎｇの位置
dim int	fcheck(2)	:// チェックされているかどうか(YES=on)
dim int	cnt(2)		:// 持ち駒の数
int	ax,ay			:// アンパッサン座標

// 手戻し用ワークエリア
enum
	MDAN=3	:// 最大記録段数（今は１手戻しまで）
endenum
int	dan		:// 記録段数
dim int	masu0(MDAN,WW,WW)	:// ます状態
dim int	ax0(MDAN),ay0(MDAN)	:// アンパッサン座標
dim int	mscol(2,WW,WW)		:// ます色

int	px,py,ppx,ppy	:// 今のポイント、前のポイント
int	fsel,bx,by		:// セレクト中フラグ,セレクト中ます
int	tm,tm0			:// セレクト中ます点滅時間
int	selCol			:// セレクト中ます枠色
int	fchk			:// チェックメイトます内フラグ
int	fmodosi			:// １手戻します内フラグ
int	fgote,f			:// 先手(0)/後手(FSEN,1)フラグ
int	te				:// 手数
int	xx,yy			:// マウス座標
int	c,cc			:// マス情報
int	fg				:// 相手の駒を取ったかどうか

dim int	touchNo(20)	:// タッチエリア番号 / touch area number
int touchallcnt =0	:// タッチエリア数   / the number of touch area

// 関数の第2以降リターン値返す用
int	ret2
//

	// 画面初期設定
	str FONT="Migu-1M-Regular"
	enum
		FNTSZ=24
	endenum
	int fzenhan
	font(FONT,FNTSZ):// 先にフォントを設定してからwidth()を設定する
	width(64,,fzenhan)
	int xw=1:// 全角１文字の桁数 / zenkaku 1 character columus
	if fzenhan then xw=2
	//
	vpriority(TPAGE,GPAGE0,GPAGE1,GPAGE2,GPAGE3)
	vpage(B_TPAGE+B_GPAGE0+B_GPAGE1)
	apage(GPAGE0)
	bitmapOpen(0,0,YES):bitmapWipe():// グラフィックをビットマップで扱う必要があるため、内部ワークを使って確保
 	keyRepeatTime(-1,0):// リピートなし / repeat none
	setTouchArea(0,0,0,0,0,YES):// これがないと任意のタッチ座標を得られない / Need for getting any touch coordinate

	// メインルーチン
	repeat
		// 初期化
		InitDisp()

		ax =-1
		ppx=-1	:// 一番最初の印
		te =0
		dan=0
		f  =0	:// これはYES/NOではない
		fgote=0	:// これはYES/NOではない
		fcheck(0)=NO:fcheck(1)=NO
		fmodosi=NO
		fsel=NO
		fchk=NO
		// 最初の駒の数;マス横幅*2行分
		cnt(0)=WW*2
		cnt(1)=WW*2

		// タッチエリア登録
		setMainTouchKey()
		kbclr()

		dim float tpoint(2)={0.0,0.0}
		int touchCnt
		int key

		while YES
			// タッチはマウス移動＋左クリックである
			// したがって、iOS版ではマウスカーソルだけ移動状態はありえない
			
			// タッチされる/キーが押されるまで待つ
			key=KEY_NONE
			while getTouchStatus()=TOUCH_NONE
				key=inkey():kbclr()
				if key<>KEY_NONE then {
					// タッチされてなくてもキーが押されていればループを抜ける
					// locate(0,26):print "key=";chr$(key)
					break:
				}
				if fsel then {:// １ますセレクト中
					tm=time()
					if tm<>tm0 then {:// １秒に１回変更
						selCol=BlinkMasu(bx,by,selCol)
						tm0=tm
					}
				}
			endwhile
			
			locate(0,MES_Y):print chr$(DISP_CTRL_CLEAR_EOS)

			// キー判定
			// continueとbreakを使うためswitchにしない
//			if key=KEY_CANCEL then {:// キャンセルボタンを押した
//				locate(0,27):print "キャンセル;fsel=";fsel
//				if fsel then {:// セレクト中はそれを解除
//					DispMasu(bx,by,0)
//					EraseMove() :// 動けるますを消す
//					fsel=NO
//				}
//				continue
//			}

			if key=KEY_DRAW then {
				DispDraw(1)
				if te>=2 then PrintDraw()
				break:// 終了
			}
			if key=KEY_CHECK then {
				// チェックメイト判定
				if fcheck(f) then {
					DispCheck(1)
					// 勝ったのはfgote xor FSEN
					fgote = fgote xor FSEN
					f = f xor 1
					beep()
					DispSenteGote(fgote)
					PrintCheckMate(fgote)
					break:// 終了
				}
			}
			if key=KEY_MODOSHI then {
				//locate(0,25):print chr$(DISP_CTRL_CLEAR_EOS)
				// １手戻し判定
				if fmodosi then {
					// 戻し可能な時のみ
					//locate(0,MES_Y+1):print "1手戻し";
					DispModosi(1)
					dan=Modosi1(dan)
					fcheck(f)=NO :// 自分のチェック解除
					DispModosi(-1)
					fmodosi=NO :// 戻せるのは１手だけ
					fsel=NO
					fgote = fgote xor FSEN
					f = f xor 1
					DispSenteGote(fgote)
					ModosiTe() :// 手表示戻し
					continue
				}
			}

			touchCnt=getTouchPoint(tpoint)
			xx=tpoint(0)
			yy=tpoint(1)
			//locate(0,25):print xx,yy

			if not PointXY(xx,yy) then {
				// マス内
				if px<>ppx or py<>ppy then {:// 前の座標と違う時
					// 今の座標をセレクト
					DispMasu(px,py,1)
					//locate(0,26):print "前の座標と違う:前=";ppx,ppy;"/新=";px,py
					if ppx<>-1 then {:// 一番最初以外
						// 前のポイントを戻す
						//locate(0,27):print "前のポイントを戻す"
						DispMasu(ppx,ppy,0)
						if fsel then {:// セレクト中は戻してから枠表示
							if (ppx=bx and ppy=by) or (px=bx and py=by) then {
								// 前の場所がセレクトされていた時
								// セレクトしているところに戻った時
								// -> その場も再表示
								selCol=BlinkMasu(bx,by,selCol)
							}
						}
					}
					ppx=px
					ppy=py
				}

				// セレクトます情報
				cc=masu(py,px)
			
				if not fsel then {:// 今はセレクトしていない＝初期選択
					//locate(0,28):print "初期選択:";
					c=cc and ALCHR
					if c>=(PAWN or fgote) and c<=(KING or fgote) then {:// 今操作出来る駒である
						beep()
						if EnableMove(px,py)=0 then {
							// この駒は動けない -> セレクトしない
							locate(0,MES_Y):print localizedString("このコマは動けない","Can't move this piece")
							beep()
						} else {:// 動けるのでセレクト
							//print "動けるので選択"
							fsel=YES
							goto "SEL"
						}
					} else {
						// 今操作できない または 駒のないところ
						//print "今操作できないか駒がない"
						if c then {
							// 今操作出来ない駒
							locate(0,MES_Y):print localizedString("このコマは今操作できない","Can't move this piece now")
							beep()
						}
					}
					goto "BTN"
				}

				// 現在セレクト中
				//locate(0,28):print "現在選択中"
				if bx=px and by=py then {
					// 同じ所でもう１回押したら解除
					DispMasu(bx,by,0)
					EraseMove()
					fsel=NO
					goto "BTN"
				}:// 同じ所ではない
			
				// 何もないます/取れる駒を選んだ -> 駒を移動するかも知れない
				//locate(0,22):print "cc=$";hex$(cc);"/MOVER=$";hex$(MOVER)
				if (cc and MOVER)=0 then {:// 移動可能フラグがない
					// ここには移動出来ない
					locate(0,MES_Y):print localizedString("ここには移動できない","Can't move here.");
					beep()
					c=cc and ALCHR
					if c>=(PAWN or fgote) and c<=(KING or fgote) then {
						// 今操作出来る別の駒をセレクトした時はセレクト移動
						//print "セレクト移動"
						DispMasu(bx,by,0) :// 前のセットされている所を消す
						EraseMove() :// 前の動けるますを消す
						if EnableMove(px,py)>0 then {:// 動ける場所がある
						label "SEL"
							bx=px
							by=py
							tm0=-1
							selCol=x68rgb(0,31,15)
							goto "BTN"
						}
						// 動ける場所がない時 -> 解除
						fsel=NO
					} // 今はこの駒は操作できない
					beep()
					goto "BTN"
				}
			
				// 移動可能 -> そこに駒を移動する／駒を取る
				// (bx,by) -> (px,py)
				//locate(0,23):print "コマの移動"
				c=masu(by,bx)
				masu(py,px)=c
				masu(by,bx)=0
				
				// 移動後チェック判定：移動により自分がチェックになるかどうか
				if AllCheck(-1,-1,fgote) then {
					// チェックされる：駒の移動し直し
					PrintCheckMessage(fgote)
					Modosi1(dan+1) :// この局面の最初に戻る
					fsel=NO
					goto "BTN"
				}
				// チェックメイトも消す
				DispCheck(-1)
			
				// 移動に伴う特別処理
				yy=SpecialKoma(px,py,c,cc,fgote)
				if yy<=0 then {
					if ax>=0 then {:// アンパッサンは直後の１回だけ有効
						masu(ay,ax) = (masu(ay,ax) xor AP)
						ax=-1
					}
				}
				// 相手の駒の数を数える
				fg=0
				if (cc and CHR) or yy=2 then {
					fg=1+(yy=2)
					cnt(f xor 1)=cnt(f xor 1)-1
					if cnt(0)=1 and cnt(1)=1 then {
						// 最後に駒が１つ残ったとき、それはKingである
						// 双方King同士になったときは引き分け
						PrintTe(c,bx,by,px,py,f,fg,0,0) :// 手表示
						beep()
						PrintDraw()
						break:// 終了
					}
				}
				
				// 移動後処理
				DispMasu(bx,by,0) :// 移動元セレクト解除
				DispMasu(px,py,1) :// 移動先セレクト
				EraseMove():// 移動可能マスクリア
				fsel=NO	   :// セレクト解除
			
				// 自分のチェック表示消去
				// ここに来るときは自分のチェックは必ずはずれている
				PrintCheck(fgote xor FSEN,0) :// 相手に出ている自分に対するチェックを消す
				fcheck(f)=NO
				
				// チェック判定：相手をチェックしたかどうか
				xx=SenteGoteCheck(-1,-1,fgote)
				int yyy=0
				if yy<0 then yyy=-yy
				PrintTe(c,bx,by,px,py,f,fg,yyy,xx) :// 手表示
				if xx then {:// チェックしている
					// チェックメイト判定
					if CheckMate(fgote xor FSEN) then {
						// 相手のＫｉｎｇがチェックメイトかどうか
						// チェックメイト！
						beep()
						PrintCheckMate(fgote)
						break :// 終了
					}
					// チェックメイトではない
					fchk=YES
				} else {:// チェックでない
					// 以前チェックされていてそれが解除されたときは消す
					if fcheck(f) then DispCheck(-1)
				}
			
				// １手戻しのためますを覚えておく
				dan=RecMasu(dan)
				if not fmodosi then {
					fmodosi=YES
					DispModosi(0)
				} // 現在１手戻しがすでに表示されているので再表示しない
				
				// 先手/後手入れ替え
				fgote =fgote xor FSEN
				f = f xor 1
				DispSenteGote(fgote)
				fcheck(f)=xx :// チェックされているかどうか
				te=te+1 :// 手数
			} // if (マス内)
		label "BTN"
			// ボタンが離されるのを待つ
			while getTouchStatus()<>TOUCH_NONE
				// ここではキーチェックしない
			endwhile
		endwhile
		
		removeKeys()

		// もう一度行いますか？
		dim str menues(1)
		str title$
		if isLocalizeJapan() then {
			title$="もう一度しますか？"
			menues={"はい","いいえ"}
		} else {
			title$="Try again ?"
			menues={"Yes","No"}
		}
		switch selectMenu2(title$,menues)
			case 0
				xx=YES
				break
			case 1
				xx=NO
				break
		endswitch
	until xx=NO
	displayFunctionKey(NO):// ファンクションキーを消す
	bitmapClose()
end

//*******************************************************

func int setMainTouchKey()
// メインのタッチキー設定
	removeKeys()
	int i=0
	touchNo(inc(i))=setTouchArea( ESX, ESY, EWX, EWY,KEY_DRAW	,YES):// Draw
	touchNo(inc(i))=setTouchArea( TSX, TSY, TWX, TWY,KEY_MODOSHI,YES):// 1手戻し
	touchNo(inc(i))=setTouchArea(CHSX,CHSY,CHWX,CHWY,KEY_CHECK	,YES):// チェックメイト
	touchallcnt =i
	return(i)
endfunc

//*******************************************************
// Ｐａｗｎの変化
//*******************************************************

func int ChangePawn(f;int)
// Pawnの変化(いわゆる「成金」)
// f : 0=先手 / FSEN=後手
dim str mkoma(5)={"?","P","Rook","Bishop","Knight","Queen"}
int sx,sy,ex,ey
int i,ret
enum
	sSX=(SX-4)
	eEX=(EX+8)
	WM =(6*8+4)
	NM =(WM+4)
endenum
	// ここ専用のタッチキーを設定するので前のを解除する
	removeKeys()

	sy=CSY2:if f then sy=CSY1:// 相手の領域内に出る＝変わる場所（相手の領域）に近いところ
	ey=sy+24
	beep()
	// 変化できる駒の表示
	ret=0
	sx=sSX
	for i=ROOK to QUEEN
		ex=sx+WM
		touchNo(inc(ret))=setTouchArea(sx,sy,ex,ey,i,YES)
		filsym(sx,sy,ex,ey,x68rgb(31,31,0),x68rgb(15,16,0),x68rgb(0,0,30),x68rgb(0,25,25),mkoma(i),1)
		sx=sx+NM
	next
	touchallcnt =ret
	
	int key
	repeat
		key=inkey()
	until key<>KEY_NONE
	kbclr()
//locate(0,23):print "key=";key

//	sx=sSX+NM*key
//	ex=sx+WM
//	filsym(sx,sy,ex,ey,x68rgb(31,31,0),x68rgb(16,15,31),x68rgb(31,31, 1),x68rgb(31, 6, 6),mkoma(key),1)
	beep()
	// タッチキーを元に戻す
	setMainTouchKey()
	// 選択ますのクリア
	x68fill(sSX,sy,eEX,ey,0)
	return(key)
endfunc

//*******************************************************
// 表示
//*******************************************************

func InitDisp()
// 画面初期化
	cls()
	wipe()
	SetMasuInit()
	DispField()
	DispDraw(0)
	WakuTe()
	DispSenteGote(0)
	DispMini(-1,-1)
	RecMasu(-1) :// 初期記録
endfunc

//-------------------------------------------------------

func DispField()
// フィールドの初期表示
int x,y
str ch
//	apage(GPAGE1)
	// 外枠表示
	x68fill(FSX ,FSY ,FEX,FEY,x68rgb(30,30,26))
	x68fill(SX-1,SY-1,EX ,EY ,0)

	// ます番号表示
	gColor(&h000080ff)
	for x=0 to WW-1:// 横
		ch=utf8Chr$('a'+x)
		symbol(SX+x*MX+MX/2-4,FSY-1,ch,FONT,14) :// 上
		symbol(SX+x*MX+MX/2-4,EY   ,ch,FONT,14) :// 下
	next
	for y=0 to WW-1:// 縦
		ch=utf8Chr$('8'-y)
		symbol(FSX+2,SY+y*MY+MY/2-6,ch,FONT,14) :// 左
		symbol(EX +3,SY+y*MY+MY/2-6,ch,FONT,14) :// 右
	next
//	apage(GPAGE0)
	
	// ます表示
	for y=0 to WW-1
		for x=0 to WW-1
			DispMasu(x,y,0)
		next
	next
endfunc

//-------------------------------------------------------

func DispMasu(px;int,py;int,mode;int)
// ますの（内容）表示
// mode : 0=通常 , 1=セレクト中
int col,col2,x,y
int c,cc
str KomaFont="Hiragino Mincho ProN":// 上下反転がわかるフォントにすること。特に「N」
str zkoma
//dim str	zkoma(7)={"","Ｐ","Ｒ","Ｂ","Ｎ","Ｑ","Ｋ","？"}	:// 全角文字
dim str	Bzkoma(7)={"","♟","♜","♝","♞","♛","♚","？"}	:// 全角文字
dim str	Wzkoma(7)={"","♙","♖","♗","♘","♕","♔","？"}	:// 全角文字
//♚	BLACK CHESS KING	Unicode: U+265A, UTF-8: E2 99 9A
//♛	BLACK CHESS QUEEN	Unicode: U+265B, UTF-8: E2 99 9B
//♜	BLACK CHESS ROOK	Unicode: U+265C, UTF-8: E2 99 9C
//♝	BLACK CHESS BISHOP	Unicode: U+265D, UTF-8: E2 99 9D
//♞	BLACK CHESS KNIGHT	Unicode: U+265E, UTF-8: E2 99 9E
//♟	BLACK CHESS PAWN	Unicode: U+265F, UTF-8: E2 99 9F
//♔	WHITE CHESS KING	Unicode: U+2654, UTF-8: E2 99 94
//♕	WHITE CHESS QUEEN	Unicode: U+2655, UTF-8: E2 99 95
//♖	WHITE CHESS ROOK	Unicode: U+2656, UTF-8: E2 99 96
//♗	WHITE CHESS BISHOP	Unicode: U+2657, UTF-8: E2 99 97
//♘	WHITE CHESS KNIGHT	Unicode: U+2658, UTF-8: E2 99 98
//♙	WHITE CHESS PAWN	Unicode: U+2659, UTF-8: E2 99 99

	x=SX+px*MX
	y=SY+py*MY
	cc=masu(py,px)
	
	// 下地
	x68fill(x,y,x+MX-1,y+MY-1,mscol(mode,py,px))
	if cc and MOVER then {:// 動けるますの下地
		x68fill(x+2,y+2,x+MX-3,y+MY-3,x68rgb(31,0,0))
	}
	c=cc and CHR :// キャラクター番号
	if c then {:// 駒がある
		col =x68rgb(0,0, 0)
//		col2=x68rgb(0,0,25)
		if mode then {
			col =x68rgb(31,31,31)
//			col2=x68rgb(31,31, 1)
		}
		gColor(col)
		if cc and FSEN then {:// 上の駒は逆転する
//			gColor(col2)
			zkoma=Bzkoma(c)
		} else {:// 下の駒は正立
//			gColor(col)
			zkoma=Wzkoma(c)
		}
		int fntsz=40://36
		int ox=(MX-fntsz)/2
		symbol(x+ox,y+3,zkoma,KomaFont,fntsz)
//		// 上下判別のため下線を引く
//		int hy=y+3+fntsz
//		gColor(0,0,120,100)
//		line(x+ox,hy,x+ox+fntsz,hy,,&h3333)
		//
		// 変化したPawnには'P'の下に小さな'R''B''N''Q'/成ったコマを付ける
		if cc and PCHG then {
///			gColor(col)
			c=((cc shr BPCHG) and CHR)
			if cc and FSEN then {
//				gColor(col2)
				zkoma=Bzkoma(c)
			} else {:// 下の駒は正立
//				gColor(col)
				zkoma=Wzkoma(c)
			}
//			symbol(x+32,y+30,koma((cc shr BPCHG) and CHR),KomaFont,16)
			symbol(x+32,y+30,zkoma,KomaFont,16)
		}
//		if cc and FSEN then {:// 上の駒は逆転する
//			// iOSでは文字の回転表示ができないので、描画領域をまるごと逆転する
//			revArea(x,y,MX,MY)
//		}
	}
endfunc

func revArea(sx;float,sy;float,wx;int,wy;int)
// 領域上下反転(180度回転)
	bitmapGet(sx,sy,sx+wx-1,sy+wy-1,buf)
	// 左右反転＋上下反転:データ最後からと最初からを入れ替える（半分まで）
	int lp=wx*wy-1,p=0,c
	int x,y
	for y=0 to wy/2-1:// 半分まででよい
		for x=0 to wx-1
			c=buf(p)
			buf(p)=buf(lp)
			buf(lp)=c
			p=p+1
			lp=lp-1
		next
	next
	bitmapPut(sx,sy,sx+wx-1,sy+wy-1,buf)
	bitmapImgLoad()
endfunc

//-------------------------------------------------------

func int BlinkMasu(px;int,py;int,col;int)
// ます枠の点滅
int x,y
	// iOSのラインバターンは常にONから始まるので&h3333と&hccccは同じである
	// ゆえにパターンによる枠変化はできない
	x=SX+px*MX
	y=SY+py*MY
	MasuBox(x,y,col,&h3333)
	return(col xor &hffffff00)
endfunc

func MasuBox(x;int,y;int,col;int,ptrn;int)
// 幅2のbox
	if col=0 then col=&h000000ff:// x68での0は不透明黒
	gColor(col)
//	box(x  ,y  ,x+MX-1,y+MY-1,ptrn):// 外枠に沿わすと、クリア時に残ってしまうことがある(iOSの問題)
	box(x+1,y+1,x+MX-2,y+MY-2,ptrn)
	box(x+2,y+2,x+MX-3,y+MY-3,ptrn)
// lineWidth(2)で描けばいいのかもしれない
endfunc

//-------------------------------------------------------

func int SetMasuInit()
// ますの初期配置（色を含む）
int x,y,c,cc
	cc=0
	for y=0 to WW-1
		c=cc
		for x=0 to WW-1
			masu(y,x)=0
			// 通常時
			if c=0 then {
				mscol(0,y,x)=x68rgb(29,29,29)
			} else {
				mscol(0,y,x)=x68rgb(24,15, 0)
			}
			// 選択時（反転色）
			mscol(1,y,x)=mscol(0,y,x) xor &hffffff00
			c =c xor 1 :// １ます毎に交互に色を変える
		next
		cc = cc xor 1 :// 行毎に始める色が変わる
	next
	SetKomaInitSub(6,7,   0) :// 下の駒
	SetKomaInitSub(1,0,FSEN) :// 上の駒
endfunc

//-------------------------------------------------------

func SetKomaInitSub(y1;int,y2;int,f;int)
// 駒の初期配置サブルーチン
// f : 自分の駒種類 , 0=下 / FSEN=上
dim char base(WW)=  {2,4,3,5,6,3,4,2}
int i			 :// R,N,B,Q,K,B,N,R
	for i=0 to WW-1
		masu(y1,i)=(PAWN or f)	 :// Pawnの列
		masu(y2,i)=(base(i) or f):// Kingの列
	next
	// Kingの初期座標
	if f<>0 then f=1 else f=0
	kx(f)=WW/2 :// 真ん中
	ky(f)=y2
endfunc

//*******************************************************
// 駒の移動
//*******************************************************

func int PointXY(xx;int,yy;int)
// グラフィック座標からポイントを計算する
// px,pyはグローバル変数
	if xx<SX or yy<SY then return(YES) :// 範囲外
	px=((xx-SX)/MX)
	py=((yy-SY)/MY)
	return(px>=WW or py>=WW) :// 範囲外=YES
endfunc

//-------------------------------------------------------

func int SpecialKoma(px;int,py;int,c;int,cc;int,f;int)
// 駒の移動に伴う特別処理
// cc=移動先情報 , c=移動元情報
// f : 0=先手 / FSEN=後手
// リターン値：0=アンパッサン無し , 1=アンパッサン可能性あり , 2=アンパッサンした , -1=キャスリングした
int ff,ppy
	if c=(PAWN or f) then {:// 一番最初のPawnだけの特別処理
		masu(py,px) = (masu(py,px) or SND) :// ２回目以降の印
		ppy=WW-4:if f then ppy=3
		if py=ppy then {:// ２つ動いたとき
			cc=c xor FSEN :// 相手のPawnキャラクター
			// 移動先の隣に相手のPawnがあるか
			if px>0 then {
				if (masu(py,px-1) and ALCHR)=cc then {
					// アンパッサンされる可能性あり
					masu(py,px) = (masu(py,px) or AP)
					ax=px
					ay=py
					return(1)
				}
			}
			if px<WW-1 then {
				if (masu(py,px+1) and ALCHR)=cc then {
					// アンパッサンされる可能性あり
					masu(py,px) = (masu(py,px) or AP)
					ax=px
					ay=py
					return(1)
				}
			}
		}
		return(0)
	}
	if c=(ROOK or f) then {:// 一番最初のRookだけの特別処理
		masu(py,px) = (masu(py,px) or SND) :// ２回目以降の印
		return(0)
	}
	if (c and CHR)=KING then {:// Kingのとき：座標記録
		ff=0:if f=FSEN then ff=1
		kx(ff)=px
		ky(ff)=py
	}
	if c=(KING or f) then {:// 一番最初のKingだけの特別処理
		masu(py,px) = (masu(py,px) or SND) :// ２回目以降の印
		if cc and CSMV then {:// キャスリング
			return(Castling(px,py))
		}
		return(0)
	}
	// ２手目以降のＰａｗｎの特別処理
	if c=(PAWN or f or SND) then {
		ppy=0:if f then ppy=WW-1
		if py=ppy then {:// 端まで行ったPawnの特別処理
			masu(py,px) = (masu(py,px) or (ChangePawn(f) shl BPCHG)) :// Queen等に変わる
		}
		if cc and APMV then {:// アンパッサン
			masu(ay,ax)=0
			DispMasu(ax,ay,0)
			ax=-1
			return(2)
		}
	}
	return(0)
endfunc


func int EraseMove()
// 移動出来るますの消し
int x,y
	for y=0 to WW-1
		for x=0 to WW-1
			if (masu(y,x) and MOVER) then {
				masu(y,x) = (masu(y,x) and notMOVER)
				DispMasu(x,y,0)
			}
		next
	next
endfunc

//-------------------------------------------------------

func int EnableMove(px;int,py;int)
// 移動出来るますの表示
// px,py : 移動する駒のある座標
// リターン値 : 移動できるますの数(0=移動できない)
int x,y,p
	// 移動できるますの認識
	MoveKoma(masu,masu(py,px),px,py)
	
	// 変わったところだけ再表示
	// 同時にその数も数える
	p=0
	for y=0 to WW-1
		for x=0 to WW-1
			if (masu(y,x) and MOVER) then {
				DispMasu(x,y,0)
				p=p+1
			}
		next
	next
	// ミニ表示
	DispMini(px,py)
	return(p)
endfunc

//-------------------------------------------------------

//func MoveKoma(masu(WW,WW);int,c;int,px;int,py;int)
func MoveKoma(masu(8,8);int,c;int,px;int,py;int)
// 移動できるますの認識
int f
	f=(masu(py,px) and FSEN):// 後手駒かどうか
	switch c and CHR
		case PAWN:
			Pawn(  masu,px,py,f)
			break
		case ROOK:
			Rook(  masu,px,py,f)
			break
		case BISHOP:
			Bishop(masu,px,py,f)
			break
		case KNIGHT:
			Knight(masu,px,py,f)
			break
		case QUEEN:
			Queen( masu,px,py,f)
			break
		case KING:
			King(  masu,px,py,f)
			break
	endswitch
endfunc

//*******************************************************
// 駒の動き：その座標に駒があることを確認の上呼び出すこと
// 動けるmasu(y,x)のMV/CSMVを立てる
// とれる駒のあるmasu(y,x)にGTMVを立てる（ここにも動ける）
//*******************************************************

func int SetSub(m;int,f;int)
// そのポイントに駒が置けるか／駒がとれるかを記録する
// f : 自分の駒種類 , 0=下 / FSEN=上
// リターン値:mの変更値
// ret2=駒（0なら駒無し）;ret2はグローバル変数
	// そこには他の駒があるか
	ret2=(m and ALCHR)
	if ret2=0 then {
		// ないので移動できる
		m=(m or MV)
	} else {
		// ある
		if (ret2>=(PAWN or (FSEN xor f))) and (ret2<=(KING or (FSEN xor f))) then {
			m=(m or GTMV)
		}
	}
	return(m)
endfunc

//-------------------------------------------------------

//func Pawn(masu(WW,WW);int,px;int,py;int,f;int)
func Pawn(masu(8,8);int,px;int,py;int,f;int)
// Ｐａｗｎの動き
// 一番最初は２つまで動ける（１つも可。ただし駒の飛び越えは出来ない）
// その後は１つづつ
// 一番端まで行けば'R''B''N''Q'に成る
// 隣にアンパッサン可能な駒があれば斜め移動＆駒取り可能(APMV)
// f : 自分の駒種類 , 0=下 / FSEN=上
int dr
int c
	c=masu(py,px)
	if c and PCHG then {:// このPawn変化している
		MoveKoma(masu,c shr BPCHG,px,py)
		return
	}
	if f then dr=1 else dr=-1:// 方向フラグ
	if py+dr>=0 and py+dr<=WW-1 then {
		if masu(py+dr,px)=NONE then {
			masu(py+dr,px) = (masu(py+dr,px) or MV)
			if (c and SND)=0 then {:// 最初：２歩飛ばし
				if masu(py+2*dr,px)=NONE then {
					// 最初は絶対に領域内
					masu(py+2*dr,px) = (masu(py+2*dr,px) or MV)
				}
			}
		}
		if px>0 then {
			if (masu(py,px-1) and AP) then {:// アンパッサン
				masu(py+dr,px-1) = (masu(py+dr,px-1) or (GTMV or APMV))
				// 斜め前は必ず空き
			} else {
				c=masu(py+dr,px-1) and ALCHR
				if c>=(PAWN or (FSEN xor f)) and c<=(KING or (FSEN xor f)) then {
					masu(py+dr,px-1) = (masu(py+dr,px-1) or GTMV)
				}
			}
		}
		if px<WW-1 then {
			if (masu(py,px+1) and AP) then {:// アンパッサン
				masu(py+dr,px+1) = (masu(py+dr,px+1) or (GTMV or APMV))
				// 斜め前は必ず空き
			} else {
				c=(masu(py+dr,px+1) and ALCHR)
				if c>=(PAWN or (FSEN xor f)) and c<=(KING or (FSEN xor f)) then {
					masu(py+dr,px+1) = (masu(py+dr,px+1) or GTMV)
				}
			}
		}
	}
endfunc

//-------------------------------------------------------

//func King(masu(WW,WW);int,px;int,py;int,f;int)
func King(masu(8,8);int,px;int,py;int,f;int)
// Ｋｉｎｇの動き
// 全方向に１つ動ける
// キャスリング：King and Rookが一度も動いていない時でかつRookとKingの間に
//   何もない時、Queen側のKingはBishopの位置に来てKnightはQueenの位置に来る。
//　 逆のKingはRookの隣に移動しRookはKingの逆隣に移動する。
//   ただしKingがチェックされているとき、途中移動するますの１つでも
//   敵に狙われているときはキャスリング出来ない。
// f : 自分の駒種類 , 0=下 / FSEN=上
int x,y
int ff
	ff=0:if f=FSEN then ff=1
	if (fcheck(ff)=0) and (masu(py,px) and SND)=0 then {
		// 一番最初のKingとRookのみ：キャスリング
		while (masu(py,0)=(ROOK or f)) and ((masu(py,1)+masu(py,2)+masu(py,3))=0)
			// 途中が狙われていないかどうかチェック
			if AllCheck(0,py,f) then break
			if AllCheck(1,py,f) then break
			if AllCheck(2,py,f) then break
			if AllCheck(3,py,f) then break
			// 左:Queen側のキャスリング
			masu(py,2) = masu(py,2) or CSMV
			break
		endwhile
		while (masu(py,7)=(ROOK or f)) and ((masu(py,6)+masu(py,5))=0)
			// 途中が狙われていないかどうかチェック
			if AllCheck(7,py,f) then break
			if AllCheck(6,py,f) then break
			if AllCheck(5,py,f) then break
			// 右のキャスリング
			masu(py,6) = masu(py,6) or CSMV
			break
		endwhile
	}
	// 普通の動き
	for y=-1 to 1
		if py+y<0 or py+y>WW-1 then continue
		for x=-1 to 1
			if px+x>=0 and px+x<=WW-1 then {
				masu(py+y,px+x)=SetSub(masu(py+y,px+x),f)
			}
		next
	next
endfunc

//-------------------------------------------------------

//func Queen(masu(WW,WW);int,px;int,py;int,f;int)
func Queen(masu(8,8);int,px;int,py;int,f;int)
// Ｑｕｅｅｎの動き
// 全方向に全部動ける
// ただし駒の飛び越えは出来ない
// f : 自分の駒種類 , 0=下 / FSEN=上
	Rook(  masu,px,py,f)
	Bishop(masu,px,py,f)
endfunc

//-------------------------------------------------------

//func Bishop(masu(WW,WW);int,px;int,py;int,f;int)
func Bishop(masu(8,8);int,px;int,py;int,f;int)
// Ｂｉｓｈｏｐの動き
// 斜めに全部動ける
// ただし駒の飛び越えは出来ない
// 自分を中心とした斜め方向をサーチする
// f : 自分の駒種類 , 0=下 / FSEN=上
int x,y
	// 左斜め上
	x=px
	y=py
	while x>0 and y>0
		y=y-1
		x=x-1
		masu(y,x)=SetSub(masu(y,x),f)
		if ret2 then break
	endwhile
	// 右斜め下
	x=px
	y=py
	while x<WW-1 and y<WW-1
		y=y+1
		x=x+1
		masu(y,x)=SetSub(masu(y,x),f)
		if ret2 then break
	endwhile
	// 右斜め上
	x=px
	y=py
	while x<WW-1 and y>0
		y=y-1
		x=x+1
		masu(y,x)=SetSub(masu(y,x),f)
		if ret2 then break
	endwhile
	// 左斜め下
	x=px
	y=py
	while x>0 and y<WW-1
		y=y+1
		x=x-1
		masu(y,x)=SetSub(masu(y,x),f)
		if ret2 then break
	endwhile
endfunc

//-------------------------------------------------------

//func Rook(masu(WW,WW);int,px;int,py;int,f;int)
func Rook(masu(8,8);int,px;int,py;int,f;int)
// Ｒｏｏｋの動き
// 縦横に全部動ける
// ただし駒の飛び越えは出来ない
// 自分を中心とした縦横方向をサーチする
// f : 自分の駒種類 , 0=下 / FSEN=上
int x,y
	// 上
	y=py
	while y>0
		y=y-1
		masu(y,px)=SetSub(masu(y,px),f)
		if ret2 then break
	endwhile
	// 下
	y=py
	while y<WW-1
		y=y+1
		masu(y,px)=SetSub(masu(y,px),f)
		if ret2 then break
	endwhile
	// 右
	x=px
	while x<WW-1
		x=x+1
		masu(py,x)=SetSub(masu(py,x),f)
		if ret2 then break
	endwhile
	// 左
	x=px
	while x>0
		x=x-1
		masu(py,x)=SetSub(masu(py,x),f)
		if ret2 then break
	endwhile
endfunc

//-------------------------------------------------------

//func Knight(masu(WW,WW);int,px;int,py;int,f;int)
func Knight(masu(8,8);int,px;int,py;int,f;int)
// Ｋｎｉｇｈｔの動き
// 全方向に桂馬
// f : 自分の駒種類 , 0=下 / FSEN=上
int y
	y=py-2: if y>=0		then KingSub1(masu,px,y,f)
	y=py-1:	if y>=0		then KingSub2(masu,px,y,f)
	y=py+1:	if y<=WW-1	then KingSub2(masu,px,y,f)
	y=py+2:	if y<=WW-1	then KingSub1(masu,px,y,f)
endfunc

//func KingSub1(masu(WW,WW);int,px;int,y;int,f;int)
func KingSub1(masu(8,8);int,px;int,y;int,f;int)
	if px-1>=0		then masu(y,px-1)=SetSub(masu(y,px-1),f)
	if px+1<=WW-1	then masu(y,px+1)=SetSub(masu(y,px+1),f)
endfunc

//func KingSub2(masu(WW,WW);int,px;int,y;int,f;int)
func KingSub2(masu(8,8);int,px;int,y;int,f;int)
	if px-2>=0		then masu(y,px-2)=SetSub(masu(y,px-2),f)
	if px+2<=WW-1	then masu(y,px+2)=SetSub(masu(y,px+2),f)
endfunc

//-------------------------------------------------------

func int Castling(px;int,py;int)
// キャスリング：RookとKingの入れ替え
int x1,x2,ret
	// masu(py)[x1からx2への移動]
	if px=2 then {:// Queenｻｲﾄﾞ
		x2=3
		x1=0
		ret=-2
	} else {:// px=6
		x2=5
		x1=7
		ret=-1
	}
	masu(py,x2)=masu(py,x1)
	masu(py,x1)=0
	DispMasu(x2,py,0)
	DispMasu(x1,py,0)
	return(ret)
endfunc

//*******************************************************

func DispMini(px;int,py;int)
// 駒移動可能位置ミニ表示
// ここでの表示では特殊動き（キャスリング／アンパッサン）は入らない
int c
dim int masu2(WW,WW)
int x,y,xx,yy
	// 純粋に移動可能コマを得るため、コマのないマス目を作る
	for y=0 to WW-1
		for x=0 to WW-1:
			masu2(y,x)=0
		next
	next
	if px>=0 then {
		// 自分の駒だけ置く
		c=masu(py,px)
		masu2(py,px)=c
		MoveKoma(masu2,c,px,py)
		masu2(py,px)=(masu2(py,px) or MV):// 自分のポイントも表示する
	}:// px<0は初期表示(マス目のみ表示)
	// ミニます表示
	for y=0 to WW-1
		for x=0 to WW-1
			xx=MSX+x*MMX
			yy=MSY+y*MMY
			x68box(xx,yy,xx+MMX-1,yy+MMY-1,x68rgb(31,31,31),&hffff)
			int col=x68rgb(0,0,0)
			if (masu2(y,x) and MOVER) then col=x68rgb(31,0,0)
			x68fill(xx+1,yy+1,xx+MMX-1,yy+MMY-1,col)
		next
	next
	x68box(MSX,MSY,xx+MMX,yy+MMY,x68rgb(31,31,31),&hffff)
endfunc

//*******************************************************
// チェック判定
// 指定した駒が次の１手でＫｉｎｇをとれるかどうか
// リターン値：0=とれない , 1=とれる
// px,py : チェックを調べる駒の座標
//*******************************************************

func int AllCheck(px;int,py;int,f;int)
// 自分のＫｉｎｇがチェックされるかどうかを調べる
// 相手の駒全部からのチェックを調べる
// f : 自分の駒種類 , 0=下 / FSEN=上
// px,py : Ｋｉｎｇを置くべき位置（-1なら現在の位置）
int x,y,ret=NO,ff
int c,c1,c2
	ff=0:if f<>0 then ff=1
	if px<>-1 then {:// 座標変更
		c1=masu(ky(ff),kx(ff))
		c2=masu(py,px)
		masu(ky(ff),kx(ff))=0
		masu(py,px)=c1
	}
	for y=0 to WW-1
		if ret then break
		for x=0 to WW-1
			c=masu(y,x) and ALCHR
			if c>=(PAWN or (FSEN xor f)) and c<=(KING or (FSEN xor f)) then {
				// 相手の駒があった
				ret=CheckKoma(c,x,y,f)
				if ret then break
			}
		next
	next
	if px<>-1 then {:// 変更した座標を戻す
		masu(ky(ff),kx(ff))=c1
		masu(py,px)=c2
	}
	return(ret)
endfunc

//-------------------------------------------------------

func int CheckKoma(c;int,x;int,y;int,f;int)
// 相手からのチェックを調べる
// f : 自分の駒種類 , 0=下 / FSEN=上

int ret
	f=f xor FSEN :// 相手からのチェックを調べるから
	switch c and CHR
		case PAWN  :ret=ChkPawn(  x,y,f):break
		case ROOK  :ret=ChkRook(  x,y,f):break
		case BISHOP:ret=ChkBishop(x,y,f):break
		case KNIGHT:ret=ChkKnight(x,y,f):break
		case QUEEN :ret=ChkQueen( x,y,f):break
		case KING  :ret=ChkKing(  x,y,f):break
	endswitch
	return(ret)
endfunc

//-------------------------------------------------------

func int CheckMate(f;int)
// チェックメイト判定
// 自分のＫｉｎｇをチェックメイト判定
// f : 自分の駒種類 , 0=下 / FSEN=上
int x,y,mv=0,ch=0,ff
int px,py
int c
	ff=0:if f=FSEN then ff=1
	// Ｋｉｎｇの動けるすべての位置に付いてチェック判定する
	for y=-1 to 1
		py=ky(ff)+y
		if py<0 or py>WW-1 then continue
		for x=-1 to 1
			px=kx(ff)+x
			if px<0 or px>WW-1 then continue
			c=masu(py,px) and ALCHR
			if c=NONE or (c>=(PAWN or (FSEN xor f)) and c<=(KING or (FSEN xor f))) then {
				mv=mv+1 :// 移動可能な場所の数
				ch=ch+AllCheck(px,py,f) :// チェックされる場所の数
			}
		next
	next
	if mv=ch then {
		// すべての移動可能な位置でチェックされ、
		// かつ、持ち駒の数がそれより少ないときは絶対チェックメイト
		//（他の駒でチェックメイトを防ぎきれない）
		if mv>cnt(ff) then return(YES)
	}
	return(NO)
endfunc

//-------------------------------------------------------

func int ChkPawn(px;int,py;int,f;int)
// Ｐａｗｎ
// f : 自分の駒種類 , 0=下 / FSEN=上
int dr
int ck
	//locate(0,22):print px,py
	if (masu(py,px) and PCHG) then {:// このPawnは変化している
		return(CheckKoma(masu(py,px) shr BPCHG,px,py,f))
	}
	if f then dr=1 else dr=-1:// 方向フラグ
	if py+dr>=0 and py+dr<=WW-1 then {
		ck=(KING or (FSEN xor f))
//		if px>0    and (masu(py+dr,px-1) and ALCHR)=ck then return(YES)
//		if px<WW-1 and (masu(py+dr,px+1) and ALCHR)=ck then return(YES)
		// Cの&&は比較評価なので、&&前の評価が偽になれば後の式は実行されないが、
		// X-BASICではandはビット演算子なので前後の式が両方評価されたあとにandされる
		// すなわち、X-BASICのandはCの&に相当するのである
		// だから、評価順が重要である。
		if px>0 then {
			if (masu(py+dr,px-1) and ALCHR)=ck then return(YES)
		}
		if px<WW-1 then {
			if (masu(py+dr,px+1) and ALCHR)=ck then return(YES)
		}
		// アンパッサンでKingはとれないので処理しない
	}
	return(NO)
endfunc

//-------------------------------------------------------

func int ChkKing(px;int,py;int,f;int)
// Ｋｉｎｇ
// f : 自分の駒種類 , 0=下 / FSEN=上
int x,y
int ck
	// キャスリングではKingはとれないので処理しない
	ck=(KING or (FSEN xor f))
	for y=-1 to 1
		if py+y<0 or py+y>WW-1 then continue
		for x=-1 to 1
			if px+x<0 or px+x>WW-1 then continue
			if (masu(py+y,px+x) and ALCHR)=ck then return(YES)
		next
	next
	return(NO)
endfunc

//-------------------------------------------------------

func int ChkQueen(px;int,py;int,f;int)
// Ｑｕｅｅｎ
// f : 自分の駒種類 , 0=下 / FSEN=上
int ret
	ret=ChkRook(px,py,f)
	if ret then return(ret)
	return(ChkBishop(px,py,f))
endfunc

//-------------------------------------------------------

func int ChkBishop(px;int,py;int,f;int)
// Ｂｉｓｈｏｐ
// f : 自分の駒種類 , 0=下 / FSEN=上

int x,y
int c,ck
	ck=(KING or (FSEN xor f))
	// 左斜め上
	x=px
	y=py
	while x>0 and y>0
		y=y-1
		x=x-1
		c=masu(y,x) and ALCHR
		if c=ck then return(YES)
		if c then break
	endwhile
	// 右斜め下
	x=px
	y=py
	while x<WW-1 and y<WW-1
		y=y+1
		x=x+1
		c=masu(y,x) and ALCHR
		if c=ck then return(YES)
		if c then break
	endwhile
	// 右斜め上
	x=px
	y=py
	while x<WW-1 and y>0
		y=y-1
		x=x+1
		c=masu(y,x) and ALCHR
		if c=ck then return(YES)
		if c then break
	endwhile
	// 左斜め下
	x=px
	y=py
	while x>0 and y<WW-1
		y=y+1
		x=x-1
		c=masu(y,x) and ALCHR
		if c=ck then return(YES)
		if c then break
	endwhile
	return(NO)
endfunc

//-------------------------------------------------------

func int ChkRook(px;int,py;int,f;int)
// Ｒｏｏｋ
// f : 自分の駒種類 , 0=下 / FSEN=上

int x,y
int c,ck
	ck=(KING or (FSEN xor f))
	// 上
	y=py
	while y>0
		y=y-1
		c=masu(y,px) and ALCHR
		if c=ck then return(YES)
		if c<>0 then break
	endwhile
	// 下
	y=py
	while y<WW-1
		y=y+1
		c=masu(y,px) and ALCHR
		if c=ck then return(YES)
		if c<>0 then break
	endwhile
	// 右
	x=px
	while x<WW-1
		x=x+1
		c=masu(py,x) and ALCHR
		if c=ck then return(YES)
		if c<>0 then break
	endwhile
	// 左
	x=px
	while x>0
		x=x-1
		c=masu(py,x) and ALCHR
		if c=ck then return(YES)
		if c<>0 then break
	endwhile
	return(NO)
endfunc

//-------------------------------------------------------

func int ChkKnight(px;int,py;int,f;int)
// Ｋｎｉｇｈｔ
// f : 自分の駒種類 , 0=下 / FSEN=上
int y,ret
int ck
	ck=(KING or (FSEN xor f))
	y=py-2
	if y>=0 then {
		if ChkKingSub1(px,y,ck) then return(YES)
	}
	y=py-1
	if y>=0 then {
		if ChkKingSub2(px,y,ck) then return(YES)
	}
	y=py+1
	if y<=WW-1 then {
		if ChkKingSub2(px,y,ck) then return(YES)
	}
	y=py+2
	if y<=WW-1 then {
		if ChkKingSub1(px,y,ck) then return(YES)
	}
	return(NO)
endfunc

func int ChkKingSub1(px;int,y;int,ck;int)
	if px-1>=0	  then if (masu(y,px-1) and ALCHR)=ck then return(YES)
	if px+1<=WW-1 then if (masu(y,px+1) and ALCHR)=ck then return(YES)
	return(NO)
endfunc

func int ChkKingSub2(px;int,y;int,ck;int)
	if px-2>=0	  then if (masu(y,px-2) and ALCHR)=ck then return(YES)
	if px+2<=WW-1 then if (masu(y,px+2) and ALCHR)=ck then return(YES)
	return(NO)
endfunc

//*******************************************************
// Draw処理
//*******************************************************

func int DispDraw(mode;int)
// Drawマスの表示
// mode : 0=off , 1=on
int col1,col2
	col1=x68rgb(0,0,31) :// ベース
	col2=x68rgb(31,0,0) :// 文字
	if mode then {
		col1=x68rgb(31,0, 0)
		col2=x68rgb( 0,0,31)
	}
	filsym(ESX,ESY,EEX,EEY,x68rgb(18,31,28),col1,x68rgb(5,0,15),col2,localizedString("ドロー","Draw"),2)
endfunc

//-------------------------------------------------------

func PrintDraw()
// 引き分け
	x68fill(DSX  ,DSY  ,DEX  ,DEY  ,x68rgb(25, 1, 0))
	x68fill(DSX+2,DSY+2,DEX-2,DEY-2,x68rgb( 1,25,25))
	symbol2(DSX+13,DSY+14,localizedString("引き分け","Ｄｒａｗ"),FONT,16,x68rgb(0,0,15),x68rgb(31,30,29))
endfunc

//*******************************************************
// １手戻し処理
//*******************************************************

func int DispModosi(mode;int)
// １手戻しマスの表示
// mode : 0=off , 1=on , -1=表示消し
int col1,col2
	if mode<0 then {
		x68fill(TSX,TSY,TEX,TEY,0)
	} else {
		col1=x68rgb(0,0,31) :// ベース
		col2=x68rgb(31,0,0) :// 文字
		if mode then {
			col1=x68rgb(31,0,0)
			col2=x68rgb(0,0,31)
		}
		filsym(TSX,TSY,TEX,TEY,x68rgb(18,31,28),col1,col2,x68rgb(28,28,28),localizedString("１手戻し","UNDO"),1)
	}
endfunc

//-------------------------------------------------------

func int RecMasu(dan;int)
// １手戻しのためのますを覚える
int x,y
	dan=dan+1
	if dan>=MDAN then dan=0
	for y=0 to WW-1
		for x=0 to WW-1
			masu0(dan,y,x)=masu(y,x)
		next
	next
	// アンパッサン座標記録
	ax0(dan)=ax
	ay0(dan)=ay
	return(dan)
endfunc

//-------------------------------------------------------

func int Modosi1(dan;int)
// １手戻し
int x,y
	// ます戻し＆表示
	dan=dan-1
	if dan<0 then dan=MDAN-1
	for y=0 to WW-1
		for x=0 to WW-1
			if masu(y,x)<>masu0(dan,y,x) then {
				// 変化したところだけ再表示
				masu(y,x)=masu0(dan,y,x)
				DispMasu(x,y,0)
			}
		next
	next
	// アンパッサン座標戻し
	ax=ax0(dan)
	ay=ay0(dan)
	return(dan)
endfunc

//*******************************************************
// 先手後手処理
//*******************************************************

func DispSenteGote(f;int)
// 先手/後手マスの表示
// f : 0=先手 / FSEN=後手
	// この先手・後手の英語表記はチェス固有
	SenteGoteSub(localizedString("後手","Iron")	,GTSX,GTSY,GTEX,GTEY,f xor FSEN)
	SenteGoteSub(localizedString("先手","Gambit"),STSX,STSY,STEX,STEY,f)
endfunc

//-------------------------------------------------------

func SenteGoteSub(mes;str,sx;int,sy;int,ex;int,ey;int,mode;int)
// 先手・後手表示サブルーチン
// mode : 0=on , 1=off

int col1,col2,col3,col4
	col1=x68rgb(31,31, 0)
	col2=x68rgb(16,17, 0)
	col3=x68rgb( 0,28,28)
	col4=x68rgb( 0, 0,20)
	if mode then {
		col1=x68rgb( 0,0,15)
		col2=x68rgb(10,9,25)
		col3=x68rgb(17,0, 0)
		col4=x68rgb( 0,0,10)
	}
	filsym(sx,sy,ex,ey,col1,col2,col4,col3,mes,2)
endfunc

//*******************************************************
// チェック処理
//*******************************************************

func int SenteGoteCheck(px;int,py;int,f;int)
// 相手をチェック判定＆表示
// f : 0=先手 / FSEN=後手
int ret
	// 相手のＫｉｎｇをチェックしたかどうか
	ret=AllCheck(-1,-1,f xor FSEN)
	if ret then beep() :// チェックしている
	PrintCheck(f,ret)
	return(ret)
endfunc

//-------------------------------------------------------

func PrintCheck(f;int,mode;int)
// チェック表示サブルーチン
// mode : 0=チェックしていない , 1=している
// f : 0=先手 / FSEN=後手

int col
int sx=SCSX,sy=SCSY,ex=SCEX,ey=SCEY :// 先手側がチェックした
	if f then {:// 後手側がチェックした
		sx=GCSX
		sy=GCSY
		ex=GCEX
		ey=GCEY
	}
	if mode then {
		filsym(sx,sy,ex,ey,x68rgb(31,31,0),x68rgb(15,16,0),x68rgb(0,0,30) ,x68rgb(0,25,25),localizedString("チェック","Check"),1)
	} else {
		x68fill(sx,sy,ex,ey,0)
	}
endfunc

//-------------------------------------------------------

func int DispCheck(mode;int)
// チェックメイトマスの表示
// mode : 0=off , 1=on , -1=表示消し
int col1,col2
	if mode<0 then {
		x68fill(CHSX,CHSY,CHEX,CHEY,0)
	} else {
		col1=x68rgb(0,0,31) :// ベース
		col2=x68rgb(31,0,0) :// 文字
		if mode then {
			col1=x68rgb(31,0,0)
			col2=x68rgb(0,0,31)
		}
		filsym(CHSX,CHSY,CHEX,CHEY,x68rgb(18,31,28),col1,x68rgb(5,0,15),col2,localizedString("ﾁｪｯｸﾒｲﾄ","Checkmate"),1)
	}
endfunc

//-------------------------------------------------------

func PrintCheckMessage(f;int)
// チェックされます。駒を移動し直してください
// f : 0=先手 / FSEN=後手
dim str mes(0)
str title$
	beep()
	if isLocalizeJapan() then {
		title$="チェックされます"
		mes(0)="移動し直してください"
	} else {
		title$="You will be checked."
		mes(0)="Please re-move."
	}
	selectMenu2(title$,mes)
endfunc

//-------------------------------------------------------

func PrintCheckMate(f;int)
// チェックメイト：fの駒の勝ち
// f : 0=先手 / FSEN=後手
int col,col0
int sx=SWSX,sy=SWSY,ex=SWEX,ey=SWEY
str mes
	mes=localizedString("先手","Gambit")
	if f then {
		mes=localizedString("後手","Iron")
		sx=GWSX
		sy=GWSY
		ex=GWEX
		ey=GWEY
	}
	x68fill(sx  ,sy  ,ex  ,ey  ,x68rgb(31,31,0))
	x68fill(sx+2,sy+2,ex-2,ey-2,x68rgb(16,17,0))
	col =x68rgb(0,28,28)
	col0=x68rgb(0, 0, 2)
	symbol2(sx+15,sy+8 ,  mes	,FONT,16,col0,col)
	symbol2(sx+23,sy+25,localizedString("の","is")		,FONT,16,col0,col)
	symbol2(sx+15,sy+42,localizedString("勝ち","Win")	,FONT,16,col0,col)
	symbol2(sx+11,sy+61,localizedString("ﾁｪｯｸﾒｲﾄ","Checkmate"),FONT,12,col0,x68rgb(28,3,3)):// これだけ文字が小さい
endfunc

//*******************************************************
// 手の表示
//*******************************************************

func PrintTe(c;int,bx;int,by;int,px;int,py;int,f;int,fget;int,fcas;int,fchk;int)
// 手表示
// 国際表記ではなく独自表記
// f : 0=先手 / FSEN=後手
str s=""
	// 枠内スクロールアップ
	//GrCopy64K(TESX+2,TESY+12+2,TEEX-2,TEEY-2,TESX+2,TESY+2)
	bitmapGet( SCRSX,SCRSY+12	,TEEX-2,TEEY-2   ,buf):// 下から
	bitmapPut( SCRSX,SCRSY		,TEEX-2,TEEY-2-12,buf):// 上へ
	bitmapFill(SCRSX,TEEY-2-12+1,TEEX-2,TEEY-2	 ,&h000000ff):// アップしたところを消す
	bitmapImgLoad()
	
	// 表示文字列作成
	if f then {:// 後手は桁をずらして表示開始
		s=s+" "
	}
	if fcas then {:// キャスリング（一度に動く駒が２つ）
		s=s+"o-o"
		if fcas=2 then {:// Queenサイド
			s=s+"-o"
		}
	} else {:// キャスリング以外（一度に動く駒は１つ）
		// 自分の駒
		if c and PCHG then {:// 変化したPawnのとき
			// 変化した駒の名前'
			s=s+koma((c shr BPCHG) and CHR)+"'"
		} else {
			s=s+koma(c and CHR)
		}
		if fget=1 then s=s+":" :// 相手を取った印
		if fget=2 then s=s+";" :// 相手を取った印：アンパッサン
		// 移動元:行列
		s=s+utf8Chr$('a'+bx)
		s=s+utf8Chr$('8'-by)
		// 移動先:行列
		s=s+"-"
		s=s+utf8Chr$('a'+px)
		s=s+utf8Chr$('8'-py)
	}
	if fchk then s=s+"*" :// チェックの印
	// 枠内最下行に表示：font=6*12
	gColor(x68rgb(30,30,15))
	symbol(TESX+2,TEEY-12-2,s,FONT,12)
endfunc

//-------------------------------------------------------

func ModosiTe()
// １手戻しした時の手表示戻し
	// 枠内スクロールダウン
//	GrCopy64KR(TESX+2,TESY+2 ,TEEX-2,TEEY-2-12,TESX+2,TEEY-2)
	bitmapGet( SCRSX,SCRSY	 ,TEEX-2,TEEY-2-12   ,buf):// 上から
	bitmapPut( SCRSX,SCRSY+12,TEEX-2,TEEY-2		 ,buf):// 下へ
	bitmapFill(SCRSX,SCRSY	 ,TEEX-2,SCRSY+2+12-1,&h000000ff):// ダウンしたところを消す
	bitmapImgLoad()
endfunc

//-------------------------------------------------------

// 手表示の枠
func WakuTe()
	x68box(TESX-1,TESY-1,TEEX+1,TEEY+1,x68rgb(31,31,31),&hffff)
endfunc

//*******************************************************
// X68互換関数など
//*******************************************************

func int x68rgb(r;int,g;int,b;int)
	// X68のGRAMからRGB別データを読み取る
	//  54321 09876 54321 0 bit
	// %00000_00000_00000_1	輝度
	// %00000_00000_11111_1	青
	// %00000_11111_00000_0	赤
	// %11111_00000_00000_0	緑
	int col=(g shl 11)+(r shl 6)+(b shl 1)
	return (x68Color2iOSColor(col))
endfunc

func x68fill(sx;int,sy;int,ex;int,ey;int,col;int)
	if col=0 then col=&h000000ff:// x68での0は不透明黒
	gColor(col)
	fill(sx,sy,ex,ey)
endfunc

func x68box(sx;int,sy;int,ex;int,ey;int,col;int,ptrn;int)
	if col=0 then col=&h000000ff:// x68での0は不透明黒
	gColor(col)
	box(sx,sy,ex,ey,ptrn)
endfunc

//*******************************************************

func symbol2(x;int,y;int,mes;str,fnt;str,pnt;float,col0;int,col1;int)
// 影付きsymbol()
	if col0=0 then col0=&h000000ff:// x68での0は不透明黒
	gColor(col0):symbol(x+1,y+1,mes,fnt,pnt):// 影
	if col1=0 then col1=&h000000ff:// x68での0は不透明黒
	gColor(col1):symbol(x  ,y  ,mes,fnt,pnt):// 本体
endfunc

func filsym(sx;int,sy;int,ex;int,ey;int,col1;int,col2;int,col3;int,col4;int,mes;str,type;int)
// 下地付き影付き文字表示
int wx,wy,ww=16
	if type=2 then ww=24
	x68fill(sx,sy,ex,ey,col1)
	sx=sx+2
	sy=sy+2
	ex=ex-2
	ey=ey-2
	x68fill(sx,sy,ex,ey,col2)
	// センタリング表示
	wx=(ex-sx+1)-utf8StrWidth(mes)*(ww/2)
	wy=(ey-sy+1)-ww
	symbol2(sx+wx/2,sy+wy/2,mes,FONT,ww,col3,col4)
endfunc

//****************************************************************
// 汎用関数 / General functions
//****************************************************************

func removeKeys()
// 全タッチアリアの解除 / release all key touch area
	if touchallcnt>0 then {
		int i
		for i=1 to touchallcnt
			removeTouchArea(touchNo(i))
		next
		touchallcnt =0:// 登録してないの印
	}:// すでに解除済みの時は何もしない / if already released, do nothing.
endfunc

//****************************************************************
